import { View, Text, FlatList, Pressable, TextInput, TouchableOpacity, Linking, Alert, SafeAreaView, Keyboard, useWindowDimensions, Switch } from 'react-native';
import React, { useEffect, useRef, useState } from 'react';
import { resetLogItems, toggleDebugger, useDebugItems } from './utils';
import Clipboard from '@react-native-clipboard/clipboard';
import { useShowDebugger } from './utils/useShowDebugger';
import { styles } from './styles';
import { DebuggerState } from './state';
import { ListItem } from './components/ListItem';
import { DebuggerSwitcher } from './components/DebuggerSwitcher';
import { getCompleteDebugItems } from './utils/getCompleteDebugItems';
export { resetLogItems, logToDebugger, useShowDebugger, useDebugItems, toggleDebugger, getCompleteDebugItems, addLogItem } from './utils';
export { DebuggerSwitcher } from './components';

const Separator = () => /*#__PURE__*/React.createElement(View, {
  style: styles.separator
});

const EmptySection = () => /*#__PURE__*/React.createElement(View, {
  style: styles.emptyWrapper
}, /*#__PURE__*/React.createElement(Text, {
  style: styles.emptyText
}, "No events captured yet"));

const keyExtractor = _ref => {
  let {
    id
  } = _ref;
  return `${id}`;
};

const renderItem = _ref2 => {
  let {
    item
  } = _ref2;
  return /*#__PURE__*/React.createElement(ListItem, {
    item: item
  });
};

const persistKey = '[QA-DEBUGGER]: Persistent items';
export const Debugger = /*#__PURE__*/React.memo(_ref3 => {
  let {
    getGlobalState,
    persistentStore,
    disableSnapshots,
    sendToDefaultValue,
    customSendRequest,
    stripPersistentData = Boolean(persistentStore),
    hideSwitcher
  } = _ref3;
  const {
    height
  } = useWindowDimensions();
  const [debugItems, addItems] = useDebugItems();
  const [show] = useShowDebugger();
  const [sendTo, setSendTo] = useState(sendToDefaultValue ?? '');
  const [forceHeight, setForceHeight] = useState(null);
  const [preserve, setPreserve] = useState(false);
  const preserveInitiated = useRef(false);
  useEffect(() => {
    DebuggerState.isMounted = true;
    return () => {
      DebuggerState.isMounted = false;
    };
  }, []);

  const getPersistValue = async () => {
    if (persistentStore) {
      const value = await persistentStore.get(persistKey);

      if (value) {
        try {
          const pItems = JSON.parse(value);
          const itemsToAdd = Array.isArray(pItems) ? pItems : [];
          addItems(...itemsToAdd);
          preserveInitiated.current = true;
          persistentStore.remove(persistKey);
          setPreserve(true);
        } catch {
          preserveInitiated.current = true;
        }
      }
    }
  };

  useEffect(() => {
    getPersistValue(); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  useEffect(() => {
    DebuggerState.getGlobalState = getGlobalState;
    DebuggerState.areSnapshotsEnabled = Boolean(!disableSnapshots && getGlobalState);
  }, [getGlobalState, disableSnapshots]);

  const handleClose = () => toggleDebugger(false);

  const handleGetGlobalState = async () => {
    try {
      if (getGlobalState) {
        Clipboard.setString(JSON.stringify(await (getGlobalState === null || getGlobalState === void 0 ? void 0 : getGlobalState())));
        Alert.alert('Global state copied to clipboard');
      }
    } catch {}
  };

  const handleCopyItems = async () => {
    try {
      const completeDebugItems = await getCompleteDebugItems(debugItems);
      const clipboardString = JSON.stringify(completeDebugItems);
      Clipboard.setString(clipboardString);
      const saved = await Clipboard.getString();

      if (saved === clipboardString) {
        Alert.alert('Items copied to clipboard');
      } else {
        Alert.alert('Failed to copy, might be too many items, try sending instead');
      }
    } catch {
      Alert.alert('Failed to copy, try sending instead');
    }
  };

  const handleSend = async () => {
    if (!sendTo) return;
    const completeDebugItems = await getCompleteDebugItems(debugItems);

    if (sendTo.includes('@') && !sendTo.includes('/')) {
      const canSendEmail = await Linking.canOpenURL(`mailto:${sendTo}`);

      if (canSendEmail) {
        Linking.openURL(`mailto:${sendTo}?subject=logs${new Date().toISOString()}&body=${JSON.stringify(completeDebugItems)}`);
      } else {
        Alert.alert('Cannot send email');
      }
    } else if (customSendRequest && typeof customSendRequest === 'function') {
      customSendRequest(sendTo, completeDebugItems);
    } else {
      // 	https://webhook.site/ab0c53e9-a21e-4b46-ab4c-02fe9b6cac93
      fetch(sendTo.trim(), {
        method: 'POST',
        body: JSON.stringify(completeDebugItems)
      }).then(() => {
        Alert.alert('Success', `Logs sent to ${sendTo.trim()}`);
      }).catch(() => {
        Alert.alert('Error', `Could not send logs to ${sendTo.trim()}`);
      });
    }
  };

  useEffect(() => {
    const openListener = Keyboard.addListener('keyboardDidShow', _ref4 => {
      let {
        endCoordinates
      } = _ref4;
      setForceHeight(height - endCoordinates.height - 24);
    });
    const hideListener = Keyboard.addListener('keyboardDidHide', () => {
      setForceHeight(null);
    });
    return () => {
      openListener.remove();
      hideListener.remove();
    };
  }, [height]);

  const toggleSwitch = () => setPreserve(v => !v);

  useEffect(() => {
    if (persistentStore && !preserve && preserveInitiated.current) {
      persistentStore.remove(persistKey);
    }
  }, [persistentStore, preserve]);
  useEffect(() => {
    if (preserve && persistentStore) {
      try {
        const last10Errors = debugItems.filter(item => item.logType === 'error' && !item.preserved).slice(-10).reduce((accum, item) => {
          return [...accum, { ...item,
            data: stripPersistentData ? JSON.stringify((item === null || item === void 0 ? void 0 : item.data) || '').slice(0, 500) : item === null || item === void 0 ? void 0 : item.data,
            preserved: true,
            snapshot: undefined,
            snapshotReadable: undefined
          }];
        }, []);
        persistentStore.set(persistKey, JSON.stringify(last10Errors));
      } catch {}
    }
  }, [debugItems, persistentStore, preserve, stripPersistentData]);
  return /*#__PURE__*/React.createElement(React.Fragment, null, show ? /*#__PURE__*/React.createElement(SafeAreaView, {
    style: [styles.wrapper, forceHeight ? {
      maxHeight: forceHeight
    } : false]
  }, /*#__PURE__*/React.createElement(View, {
    style: styles.titleWrapper
  }, /*#__PURE__*/React.createElement(Text, {
    style: styles.title
  }, "QA DEBUGGER")), /*#__PURE__*/React.createElement(FlatList, {
    keyExtractor: keyExtractor,
    contentContainerStyle: styles.flex,
    ItemSeparatorComponent: Separator,
    data: debugItems,
    ListEmptyComponent: EmptySection,
    renderItem: renderItem
  }), /*#__PURE__*/React.createElement(View, {
    style: styles.buttonSection
  }, /*#__PURE__*/React.createElement(Text, {
    style: styles.buttonText
  }, " Send to"), /*#__PURE__*/React.createElement(TextInput, {
    style: styles.sendToInput,
    value: sendTo,
    onChangeText: setSendTo
  }), Boolean(sendTo) && /*#__PURE__*/React.createElement(TouchableOpacity, {
    style: styles.sendButton,
    onPress: handleSend
  }, /*#__PURE__*/React.createElement(Text, {
    style: styles.buttonText
  }, "Send"))), Boolean(getGlobalState) && /*#__PURE__*/React.createElement(View, {
    style: styles.buttonSection
  }, /*#__PURE__*/React.createElement(Pressable, {
    onPress: handleGetGlobalState,
    style: styles.button
  }, /*#__PURE__*/React.createElement(Text, {
    style: styles.buttonText
  }, "Copy global state"))), Boolean(persistentStore) && /*#__PURE__*/React.createElement(View, {
    style: styles.buttonSection
  }, /*#__PURE__*/React.createElement(Pressable, {
    onPress: toggleSwitch,
    style: styles.button
  }, /*#__PURE__*/React.createElement(Switch, {
    style: {
      transform: [{
        scale: 0.75
      }]
    },
    onValueChange: toggleSwitch,
    value: preserve
  }), /*#__PURE__*/React.createElement(Text, {
    style: styles.buttonText
  }, "\xA0\xA0Preserve 10 last errors"))), /*#__PURE__*/React.createElement(View, {
    style: styles.buttonSection
  }, /*#__PURE__*/React.createElement(Pressable, {
    onPress: handleCopyItems,
    style: styles.button
  }, /*#__PURE__*/React.createElement(Text, {
    style: styles.buttonText
  }, "Copy items")), /*#__PURE__*/React.createElement(Pressable, {
    onPress: resetLogItems,
    style: [styles.button, styles.centralButton]
  }, /*#__PURE__*/React.createElement(Text, {
    style: styles.buttonText
  }, "Clear")), /*#__PURE__*/React.createElement(Pressable, {
    onPress: handleClose,
    style: styles.button
  }, /*#__PURE__*/React.createElement(Text, {
    style: styles.buttonText
  }, "Close")))) : null, !hideSwitcher && /*#__PURE__*/React.createElement(View, {
    style: styles.switcher
  }, /*#__PURE__*/React.createElement(DebuggerSwitcher, null)));
});
//# sourceMappingURL=index.js.map