import {
  View,
  Text,
  FlatList,
  Pressable,
  TextInput,
  TouchableOpacity,
  Linking,
  Alert,
  SafeAreaView,
  Keyboard,
  useWindowDimensions,
  Switch,
} from 'react-native';
import React, { FC, useEffect, useRef, useState } from 'react';
import { resetLogItems, toggleDebugger, useDebugItems } from './utils';
import Clipboard from '@react-native-clipboard/clipboard';
import { useShowDebugger } from './utils/useShowDebugger';
import { styles } from './styles';
import { DebuggerState } from './state';
import type { Props } from './types';
import { ListItem } from './components/ListItem';
import { DebuggerSwitcher } from './components/DebuggerSwitcher';
import { getCompleteDebugItems } from './utils/getCompleteDebugItems';

export {
  resetLogItems,
  logToDebugger,
  useShowDebugger,
  useDebugItems,
  toggleDebugger,
  getCompleteDebugItems,
  addLogItem,
} from './utils';
export { DebuggerSwitcher } from './components';

const Separator = () => <View style={styles.separator} />;

const EmptySection = () => (
  <View style={styles.emptyWrapper}>
    <Text style={styles.emptyText}>No events captured yet</Text>
  </View>
);

const keyExtractor = ({ id }: { id: string | number }) => `${id}`;

const renderItem = ({ item }: any) => <ListItem item={item} />;

const persistKey = '[QA-DEBUGGER]: Persistent items';

export const Debugger: FC<Props> = React.memo(
  ({
    getGlobalState,
    persistentStore,
    disableSnapshots,
    sendToDefaultValue,
    customSendRequest,
    stripPersistentData = Boolean(persistentStore),
    hideSwitcher,
  }) => {
    const { height } = useWindowDimensions();

    const [debugItems, addItems] = useDebugItems();
    const [show] = useShowDebugger();
    const [sendTo, setSendTo] = useState(sendToDefaultValue ?? '');
    const [forceHeight, setForceHeight] = useState<null | number>(null);
    const [preserve, setPreserve] = useState(false);
    const preserveInitiated = useRef(false);

    useEffect(() => {
      DebuggerState.isMounted = true;
      return () => {
        DebuggerState.isMounted = false;
      };
    }, []);

    const getPersistValue = async () => {
      if (persistentStore) {
        const value = await persistentStore.get(persistKey);
        if (value) {
          try {
            const pItems = JSON.parse(value);
            const itemsToAdd: any[] = Array.isArray(pItems) ? pItems : [];
            addItems(...itemsToAdd);
            preserveInitiated.current = true;
            persistentStore.remove(persistKey);
            setPreserve(true);
          } catch {
            preserveInitiated.current = true;
          }
        }
      }
    };

    useEffect(() => {
      getPersistValue();
      // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);

    useEffect(() => {
      DebuggerState.getGlobalState = getGlobalState;
      DebuggerState.areSnapshotsEnabled = Boolean(
        !disableSnapshots && getGlobalState
      );
    }, [getGlobalState, disableSnapshots]);

    const handleClose = () => toggleDebugger(false);

    const handleGetGlobalState = async () => {
      try {
        if (getGlobalState) {
          Clipboard.setString(JSON.stringify(await getGlobalState?.()));
          Alert.alert('Global state copied to clipboard');
        }
      } catch {}
    };
    const handleCopyItems = async () => {
      try {
        const completeDebugItems = await getCompleteDebugItems(debugItems);
        const clipboardString = JSON.stringify(completeDebugItems);
        Clipboard.setString(clipboardString);
        const saved = await Clipboard.getString();
        if (saved === clipboardString) {
          Alert.alert('Items copied to clipboard');
        } else {
          Alert.alert(
            'Failed to copy, might be too many items, try sending instead'
          );
        }
      } catch {
        Alert.alert('Failed to copy, try sending instead');
      }
    };

    const handleSend = async () => {
      if (!sendTo) return;

      const completeDebugItems = await getCompleteDebugItems(debugItems);
      if (sendTo.includes('@') && !sendTo.includes('/')) {
        const canSendEmail = await Linking.canOpenURL(`mailto:${sendTo}`);
        if (canSendEmail) {
          Linking.openURL(
            `mailto:${sendTo}?subject=logs${new Date().toISOString()}&body=${JSON.stringify(
              completeDebugItems
            )}`
          );
        } else {
          Alert.alert('Cannot send email');
        }
      } else if (customSendRequest && typeof customSendRequest === 'function') {
        customSendRequest(sendTo, completeDebugItems);
      } else {
        // 	https://webhook.site/ab0c53e9-a21e-4b46-ab4c-02fe9b6cac93
        fetch(sendTo.trim(), {
          method: 'POST',
          body: JSON.stringify(completeDebugItems),
        })
          .then(() => {
            Alert.alert('Success', `Logs sent to ${sendTo.trim()}`);
          })
          .catch(() => {
            Alert.alert('Error', `Could not send logs to ${sendTo.trim()}`);
          });
      }
    };

    useEffect(() => {
      const openListener = Keyboard.addListener(
        'keyboardDidShow',
        ({ endCoordinates }) => {
          setForceHeight(height - endCoordinates.height - 24);
        }
      );
      const hideListener = Keyboard.addListener('keyboardDidHide', () => {
        setForceHeight(null);
      });
      return () => {
        openListener.remove();
        hideListener.remove();
      };
    }, [height]);

    const toggleSwitch = () => setPreserve((v) => !v);

    useEffect(() => {
      if (persistentStore && !preserve && preserveInitiated.current) {
        persistentStore.remove(persistKey);
      }
    }, [persistentStore, preserve]);

    useEffect(() => {
      if (preserve && persistentStore) {
        try {
          const last10Errors = debugItems
            .filter((item) => item.logType === 'error' && !item.preserved)
            .slice(-10)
            .reduce((accum, item) => {
              return [
                ...accum,
                {
                  ...item,
                  data: stripPersistentData
                    ? JSON.stringify(item?.data || '').slice(0, 500)
                    : item?.data,
                  preserved: true,
                  snapshot: undefined,
                  snapshotReadable: undefined,
                },
              ];
            }, []);
          persistentStore.set(persistKey, JSON.stringify(last10Errors));
        } catch {}
      }
    }, [debugItems, persistentStore, preserve, stripPersistentData]);

    return (
      <>
        {show ? (
          <SafeAreaView
            style={[
              styles.wrapper,
              forceHeight ? { maxHeight: forceHeight } : false,
            ]}
          >
            <View style={styles.titleWrapper}>
              <Text style={styles.title}>QA DEBUGGER</Text>
            </View>
            <FlatList
              keyExtractor={keyExtractor}
              contentContainerStyle={styles.flex}
              ItemSeparatorComponent={Separator}
              data={debugItems}
              ListEmptyComponent={EmptySection}
              renderItem={renderItem}
            />
            <View style={styles.buttonSection}>
              <Text style={styles.buttonText}> Send to</Text>
              <TextInput
                style={styles.sendToInput}
                value={sendTo}
                onChangeText={setSendTo}
              />
              {Boolean(sendTo) && (
                <TouchableOpacity
                  style={styles.sendButton}
                  onPress={handleSend}
                >
                  <Text style={styles.buttonText}>Send</Text>
                </TouchableOpacity>
              )}
            </View>
            {Boolean(getGlobalState) && (
              <View style={styles.buttonSection}>
                <Pressable onPress={handleGetGlobalState} style={styles.button}>
                  <Text style={styles.buttonText}>Copy global state</Text>
                </Pressable>
              </View>
            )}
            {Boolean(persistentStore) && (
              <View style={styles.buttonSection}>
                <Pressable onPress={toggleSwitch} style={styles.button}>
                  <Switch
                    style={{ transform: [{ scale: 0.75 }] }}
                    onValueChange={toggleSwitch}
                    value={preserve}
                  />
                  <Text style={styles.buttonText}>
                    &nbsp;&nbsp;Preserve 10 last errors
                  </Text>
                </Pressable>
              </View>
            )}
            <View style={styles.buttonSection}>
              <Pressable onPress={handleCopyItems} style={styles.button}>
                <Text style={styles.buttonText}>Copy items</Text>
              </Pressable>
              <Pressable
                onPress={resetLogItems}
                style={[styles.button, styles.centralButton]}
              >
                <Text style={styles.buttonText}>Clear</Text>
              </Pressable>
              <Pressable onPress={handleClose} style={styles.button}>
                <Text style={styles.buttonText}>Close</Text>
              </Pressable>
            </View>
          </SafeAreaView>
        ) : null}
        {!hideSwitcher && (
          <View style={styles.switcher}>
            <DebuggerSwitcher />
          </View>
        )}
      </>
    );
  }
);
